# coding: utf-8

"""
haul.compat
~~~~~~~~~~~

This module contains imports and declarations for seamless Python 2 and
Python 3 compatibility.
"""

import sys


_version = sys.version_info

is_py2 = (_version[0] == 2)
is_py3 = (_version[0] == 3)

if is_py2:
    from urlparse import urljoin, urlparse

    str = unicode

elif is_py3:
    from urllib.parse import urljoin, urlparse

    str = str



# coding: utf-8

import cStringIO
import pprint
import sys


def pp(something):
    printer = pprint.PrettyPrinter(indent=2)
    printer.pprint(something)


def import_module(name):
    __import__(name)

    return sys.modules[name]


def module_member(name):
    mod, member = name.rsplit('.', 1)
    module = import_module(mod)

    return getattr(module, member)


def read_file(path):
    with open (path, 'r') as f:
        content = f.read()

    return content


def pack_image(self, content):
    string_io = cStringIO.StringIO(content)

    return string_io



# coding: utf-8

from collections import OrderedDict
import mimetypes
import re

from bs4 import BeautifulSoup
import requests

from . import exceptions, settings, utils


simple_url_re = re.compile(r'^https?://\[?\w', re.IGNORECASE)


class Haul(object):
    """
    Haul
    """

    def __init__(self,
                 parser=settings.DEFAULT_PARSER,
                 finder_pipeline=settings.FINDER_PIPELINE,
                 extender_pipeline=settings.EXTENDER_PIPELINE):

        self.parser = parser
        self.finder_pipeline = finder_pipeline
        self.extender_pipeline = extender_pipeline

        self.response = None # via Requests
        self.soup = None # via BeautifulSoup

        self._result = None

    def __repr__(self):
        return '<Haul [parser: %s]>' % (self.parser)

    @property
    def result(self):
        if not isinstance(self._result, HaulResult):
            self._result = HaulResult()

        return self._result

    def retrieve_url(self, url):
        """
        Use requests to fetch remote content
        """

        try:
            r = requests.get(url)
        except requests.ConnectionError:
            raise exceptions.RetrieveError('Connection fail')

        if r.status_code >= 400:
            raise exceptions.RetrieveError('Connected, but status code is %s' % (r.status_code))

        real_url = r.url
        content = r.content

        try:
            content_type = r.headers['Content-Type']
        except KeyError:
            content_type, encoding = mimetypes.guess_type(real_url, strict=False)

        self.response = r

        return content_type.lower(), content

    def parse_html(self, html):
        """
        Use BeautifulSoup to parse HTML / XML
        http://www.crummy.com/software/BeautifulSoup/bs4/doc/#specifying-the-parser-to-use
        """

        soup = BeautifulSoup(html, self.parser)

        title_tag = soup.find('title')
        self.result.title = title_tag.string if title_tag else None

        self.soup = soup

        return soup

    def start_finder_pipeline(self, *args, **kwargs):
        pipeline_input = {
            'soup': self.soup,
        }
        pipeline_output = pipeline_input.copy()

        for idx, name in enumerate(self.finder_pipeline):
            pipeline_output['pipeline_index'] = idx
            pipeline_output['pipeline_break'] = False

            if hasattr(name, '__call__'):
                finder_func = name
            else:
                finder_func = utils.module_member(name)

            output = finder_func(*args, **pipeline_output)

            if isinstance(output, dict):
                pipeline_output.update(output)

            if pipeline_output['pipeline_break']:
                break

        # remove unnecessary items
        pipeline_output.pop('pipeline_index', None)
        pipeline_output.pop('pipeline_break', None)
        pipeline_output.pop('soup', None)

        self.result.finder_image_urls = pipeline_output.get('finder_image_urls', [])

        return self.result

    def start_extender_pipeline(self, *args, **kwargs):
        pipeline_input = {
            'finder_image_urls': self.result.finder_image_urls,
        }
        pipeline_output = pipeline_input.copy()

        for idx, name in enumerate(self.extender_pipeline):
            pipeline_output['pipeline_index'] = idx
            pipeline_output['pipeline_break'] = False

            if hasattr(name, '__call__'):
                extender_func = name
            else:
                extender_func = utils.module_member(name)

            output = extender_func(*args, **pipeline_output)

            if isinstance(output, dict):
                pipeline_output.update(output)

            if pipeline_output['pipeline_break']:
                break

        # remove unnecessary items
        pipeline_output.pop('pipeline_index', None)
        pipeline_output.pop('pipeline_break', None)
        pipeline_output.pop('finder_image_urls', None)

        self.result.extender_image_urls = pipeline_output.get('extender_image_urls', [])

        return self.result

    # API
    def find_images(self, url_or_html, extend=False):
        url = None
        content = None

        try:
            is_url = simple_url_re.match(url_or_html)
        except TypeError:
            raise exceptions.InvalidParameterError('Should be a URL or HTML text')

        if is_url:
            url = url_or_html
            content_type, content = self.retrieve_url(url)
        else:
            content_type = 'text/html'
            content = url_or_html

        self.result.url = url
        self.result.content_type = content_type

        if 'text/html' in content_type:
            self.parse_html(content)

            self.start_finder_pipeline()

            if extend:
                self.start_extender_pipeline()

        elif 'image/' in content_type:
            self.result.finder_image_urls = [str(self.response.url), ]

            if extend:
                self.start_extender_pipeline()

        else:
            raise exceptions.ContentTypeNotSupported(content_type)

        return self.result


class HaulResult(object):
    """
    Result of Haul
    """

    def __init__(self):
        self.url = None
        self.content_type = None
        self.title = None
        self.finder_image_urls = []
        self.extender_image_urls = []

    def __repr__(self):
        return '<HaulResult [Content-Type: %s]>' % (self.content_type)

    @property
    def is_found(self):
        return True if len(self.finder_image_urls) > 0 else False

    @property
    def image_urls(self):
        """
        Combine finder_image_urls and extender_image_urls,
        remove duplicate but keep order
        """

        all_image_urls = self.finder_image_urls[:]
        for image_url in self.extender_image_urls:
            if image_url not in all_image_urls:
                all_image_urls.append(image_url)

        return all_image_urls

    def to_dict(self):
        return self.__dict__

    def to_ordered_dict(self):
        order_keys = (
            'url',
            'content_type',
            'title',
            'finder_image_urls',
            'extender_image_urls',
        )

        d = OrderedDict()
        for key in order_keys:
            d[key] = getattr(self, key)

        return d



# coding: utf-8

"""
haul.exceptions
~~~~~~~~~~~~~~~

This module contains the set of Haul's exceptions.
"""


class InvalidParameterError(Exception):
    """
    Invalid Parameter
    """

    def __init__(self, message):
        Exception.__init__(self)

        self.message = message

    def __repr__(self):
        return '<InvalidParameterError [message: %s]>' % (self.message)


class RetrieveError(RuntimeError):
    """
    Connection fail or HTTP status code >= 400
    """

    def __init__(self, message):
        RuntimeError.__init__(self)

        self.message = message

    def __repr__(self):
        return '<RetrieveError [message: %s]>' % (self.message)


class ContentTypeNotSupported(Exception):
    """
    ref: settings.ALLOWED_CONTENT_TYPES
    """

    def __init__(self, content_type):
        Exception.__init__(self)

        self.content_type = content_type

    def __repr__(self):
        return '<ContentTypeNotSupported [content_type: %s]>' % (self.content_type)



# coding: utf-8

import re


def original_image_extender(pipeline_index,
                            finder_image_urls,
                            extender_image_urls=[],
                            *args, **kwargs):
    """
    Example:
    http://media-cache-ec0.pinimg.com/70x/50/9b/bd/509bbd5c6543d473bc2b49befe75f4c6.jpg
    http://media-cache-ec0.pinimg.com/236x/50/9b/bd/509bbd5c6543d473bc2b49befe75f4c6.jpg
    http://media-cache-ec0.pinimg.com/736x/50/9b/bd/509bbd5c6543d473bc2b49befe75f4c6.jpg
    to
    http://media-cache-ec0.pinimg.com/originals/50/9b/bd/509bbd5c6543d473bc2b49befe75f4c6.jpg
    """

    now_extender_image_urls = []

    search_re = re.compile(r'.com/\d+x/', re.IGNORECASE)

    for image_url in finder_image_urls:
        if 'pinimg.com/' in image_url.lower():
            if search_re.search(image_url):
                extender_image_url = search_re.sub('.com/originals/', image_url, count=1)
                now_extender_image_urls.append(extender_image_url)

    output = {}
    output['extender_image_urls'] = extender_image_urls + now_extender_image_urls

    return output



# coding: utf-8

import re


def blogspot_s1600_extender(pipeline_index,
                            finder_image_urls,
                            extender_image_urls=[],
                            *args, **kwargs):
    """
    Example:
    http://1.bp.blogspot.com/-S97wTYQKbrY/UkWukhKhTKI/AAAAAAAAJ0g/fcRDiqVC8Us/s898/aaPOP+001.jpg
    to
    http://1.bp.blogspot.com/-S97wTYQKbrY/UkWukhKhTKI/AAAAAAAAJ0g/fcRDiqVC8Us/s1600/aaPOP+001.jpg
    """

    now_extender_image_urls = []

    search_re = re.compile(r'/s\d+/', re.IGNORECASE)

    for image_url in finder_image_urls:
        if 'bp.blogspot.com/' in image_url.lower():
            if search_re.search(image_url):
                extender_image_url = search_re.sub('/s1600/', image_url)
                now_extender_image_urls.append(extender_image_url)

    output = {}
    output['extender_image_urls'] = extender_image_urls + now_extender_image_urls

    return output


def ggpht_s1600_extender(pipeline_index,
                         finder_image_urls,
                         extender_image_urls=[],
                         *args, **kwargs):
    """
    Example:
    http://lh4.ggpht.com/-fFi-qJRuxeY/UjwHSOTHGOI/AAAAAAAArgE/SWTMT-hXzB4/s640/Celeber-ru-Emma-Watson-Net-A-Porter-The-Edit-Magazine-Photoshoot-2013-01.jpg
    to
    http://lh4.ggpht.com/-fFi-qJRuxeY/UjwHSOTHGOI/AAAAAAAArgE/SWTMT-hXzB4/s1600/Celeber-ru-Emma-Watson-Net-A-Porter-The-Edit-Magazine-Photoshoot-2013-01.jpg
    """

    now_extender_image_urls = []

    search_re = re.compile(r'/s\d+/', re.IGNORECASE)

    for image_url in finder_image_urls:
        if 'ggpht.com/' in image_url.lower():
            if search_re.search(image_url):
                extender_image_url = search_re.sub('/s1600/', image_url)
                now_extender_image_urls.append(extender_image_url)

    output = {}
    output['extender_image_urls'] = extender_image_urls + now_extender_image_urls

    return output


def googleusercontent_s1600_extender(pipeline_index,
                                     finder_image_urls,
                                     extender_image_urls=[],
                                     *args, **kwargs):
    """
    Example:
    https://lh6.googleusercontent.com/-T6V-utZHzbE/Ukjn-1MDOSI/AAAAAAAAA3g/H6Qcw1zt4n0/w555-h399-no/2101_aa2cac09d1c6431b8a635d61cd9c4471.jpg
    to
    https://lh6.googleusercontent.com/-T6V-utZHzbE/Ukjn-1MDOSI/AAAAAAAAA3g/H6Qcw1zt4n0/s1600/2101_aa2cac09d1c6431b8a635d61cd9c4471.jpg
    """

    now_extender_image_urls = []

    search_re = re.compile(r'/w\d+\-h\d+\-no/', re.IGNORECASE)

    for image_url in finder_image_urls:
        if 'googleusercontent.com/' in image_url.lower():
            if search_re.search(image_url):
                extender_image_url = search_re.sub('/s1600/', image_url)
                now_extender_image_urls.append(extender_image_url)

    output = {}
    output['extender_image_urls'] = extender_image_urls + now_extender_image_urls

    return output



# coding: utf-8

import cssutils


def background_image_finder(pipeline_index,
                            soup,
                            finder_image_urls=[],
                            *args, **kwargs):
    """
    Find image URL in background-image

    Example:
    <div style="width: 100%; height: 100%; background-image: url(http://distilleryimage10.ak.instagram.com/bde04558a43b11e28e5d22000a1f979a_7.jpg);" class="Image iLoaded iWithTransition Frame" src="http://distilleryimage10.ak.instagram.com/bde04558a43b11e28e5d22000a1f979a_7.jpg"></div>
    to
    http://distilleryimage10.ak.instagram.com/bde04558a43b11e28e5d22000a1f979a_7.jpg
    """

    now_finder_image_urls = []

    for tag in soup.find_all(style=True):
        style_string = tag['style']
        if 'background-image' in style_string.lower():
            style = cssutils.parseStyle(style_string)
            background_image = style.getProperty('background-image')
            if background_image:
                for property_value in background_image.propertyValue:
                    background_image_url = str(property_value.value)
                    if background_image_url:
                        if (background_image_url not in finder_image_urls) and \
                           (background_image_url not in now_finder_image_urls):
                            now_finder_image_urls.append(background_image_url)

    output = {}
    output['finder_image_urls'] = finder_image_urls + now_finder_image_urls

    return output
